/***************************************************************************
 *            AlertsManager.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef ALERTSMANAGER_H
#define ALERTSMANAGER_H

#include <queue>

#include <boost/signal.hpp>
#include <boost/bind.hpp>

#include "libtorrent/alert_types.hpp"
#include "libtorrent/peer_id.hpp"
#include "libtorrent/time.hpp"
#include "libtorrent/torrent_info.hpp"
#include "libtorrent/intrusive_ptr_base.hpp"

#include "Lince/Torrent.h"
#include "Lince/TorrentsManager.h"

namespace Lince
{

class AlertsManager;
typedef boost::intrusive_ptr<AlertsManager> AlertsManagerPtr;
	
typedef boost::signal<void (int)> intSignal;
typedef boost::signal<void (std::string, int)> stringIntSignal;
typedef boost::signal<void (std::string)> stringSignal;
typedef boost::signal<void (torrentPosition)> positionSignal;
typedef boost::signal<void (libtorrent::torrent_handle&)> torrenthandleSignal;
	
enum alertLevel { FIRST_LEVEL=1, FATAL=FIRST_LEVEL, CRITICAL, WARNING, INFO, DEBUG, NONE, LAST_LEVEL };
enum localAlertType { FIRST_TYPE, TORRENT_ADDED=FIRST_TYPE, TORRENT_ADDED_PREVIOUS_SESSION,
	TORRENT_REMOVED, QUEUED, UNQUEUED, STOPPED, STARTED, MAGNET_LINK_ADDED,
	TORRENT_METADATA_ADDED, LAST_TYPE };
	
struct local_alert
{
	localAlertType type;
	alertLevel level;
	std::string name;
	libtorrent::sha1_hash hash;
	
	local_alert() {};
	local_alert(localAlertType _at, alertLevel _al, std::string _n, libtorrent::sha1_hash _h)
	{
		type = _at;
		level = _al;
		name = _n;
		hash = _h;
	};
};

class AlertsManager: public libtorrent::intrusive_ptr_base<AlertsManager>
{
	std::queue< local_alert > local_alerts; 
	
	AlertsManager();
	
	intSignal m_signal_finished_torrent;
	torrenthandleSignal m_signal_metadata_received;
	stringIntSignal m_signal_finished_torrent_log;
	stringIntSignal m_signal_listen_failed;
	stringIntSignal m_signal_listen_succeded;
	stringIntSignal m_signal_portmap_failed;
	stringIntSignal m_signal_portmap_success;
	stringIntSignal m_signal_file_error;
	stringIntSignal m_signal_tracker_announce;
	stringIntSignal m_signal_tracker_failed;
	stringIntSignal m_signal_tracker_reply;
	stringIntSignal m_signal_tracker_warning;
	stringIntSignal m_signal_http_seed_failed;
	stringIntSignal m_signal_hash_failed;
	stringIntSignal m_signal_peer_banned;
	stringIntSignal m_signal_fastresume_rejected;
	stringIntSignal m_signal_storage_moved;
	stringIntSignal m_signal_torrent_checked;
	stringIntSignal m_signal_metadata_received_log;
	stringIntSignal m_signal_metadata_failed;

	// Local signals
	positionSignal m_signal_torrent_added;
	positionSignal m2_signal_torrent_stopped;
	positionSignal m2_signal_torrent_started;
	stringIntSignal m_signal_torrent_added_log;
	stringIntSignal m_signal_torrent_removed;
	stringIntSignal m_signal_torrent_queued;
	stringIntSignal m_signal_torrent_unqueued;
	stringIntSignal m_signal_torrent_stopped;
	stringIntSignal m_signal_torrent_started;
	stringIntSignal m_signal_magnet_link_added;
	
	stringSignal m_signal_ban_peer;
	
	
public:
	
	static AlertsManagerPtr create();
	void add_local_alert(const local_alert &);
	void check_alerts();
	
	boost::signals::connection finished_torrent_signal(intSignal::slot_function_type subscriber)
	{
		return m_signal_finished_torrent.connect(subscriber);
	};

	boost::signals::connection metadata_received_signal(torrenthandleSignal::slot_function_type subscriber)
	{
		return m_signal_metadata_received.connect(subscriber);
	};
		
	boost::signals::connection finished_torrent_signal_log(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_finished_torrent_log.connect(subscriber);
	};
		
	boost::signals::connection listen_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_listen_failed.connect(subscriber);
	};
		
	boost::signals::connection listen_succeded_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_listen_succeded.connect(subscriber);
	};
		
	boost::signals::connection portmap_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_portmap_failed.connect(subscriber);
	};
		
	boost::signals::connection portmap_success_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_portmap_success.connect(subscriber);
	};
		
	boost::signals::connection file_error_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_file_error.connect(subscriber);
	};
		
	boost::signals::connection tracker_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_tracker_failed.connect(subscriber);
	};
		
	boost::signals::connection tracker_reply_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_tracker_reply.connect(subscriber);
	};
		
	boost::signals::connection tracker_warning_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_tracker_warning.connect(subscriber);
	};
		
	boost::signals::connection http_seed_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_http_seed_failed.connect(subscriber);
	};
		
	boost::signals::connection hash_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_hash_failed.connect(subscriber);
	};
		
	boost::signals::connection peer_banned_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_peer_banned.connect(subscriber);
	};
		
	boost::signals::connection fastresume_rejected_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_fastresume_rejected.connect(subscriber);
	};
		
	boost::signals::connection storage_moved_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_storage_moved.connect(subscriber);
	};
		
	boost::signals::connection torrent_checked_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_checked.connect(subscriber);
	};
		
	boost::signals::connection torrent_added_signal(positionSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_added.connect(subscriber);
	};
		
	boost::signals::connection torrent_stopped_signal2(positionSignal::slot_function_type subscriber)
	{
		return m2_signal_torrent_stopped.connect(subscriber);
	};
		
	boost::signals::connection torrent_started_signal2(positionSignal::slot_function_type subscriber)
	{
		return m2_signal_torrent_started.connect(subscriber);
	};
		
	boost::signals::connection torrent_added_log_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_added_log.connect(subscriber);
	};
		
	boost::signals::connection torrent_removed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_removed.connect(subscriber);
	};
		
	boost::signals::connection torrent_queued_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_queued.connect(subscriber);
	};
		
	boost::signals::connection torrent_unqueued_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_unqueued.connect(subscriber);
	};
			
	boost::signals::connection torrent_stopped_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_stopped.connect(subscriber);
	};
		
	boost::signals::connection torrent_started_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_torrent_started.connect(subscriber);
	};
	
	boost::signals::connection ban_peer_signal(stringSignal::slot_function_type subscriber)
	{
		return m_signal_ban_peer.connect(subscriber);
	};
	
	boost::signals::connection magnet_added_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_magnet_link_added.connect(subscriber);
	};

	boost::signals::connection metadata_recived_log_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_metadata_received_log.connect(subscriber);
	};

	boost::signals::connection metadata_failed_signal(stringIntSignal::slot_function_type subscriber)
	{
		return m_signal_metadata_failed.connect(subscriber);
	};

	~AlertsManager();
};

} // Namespace

#endif
