/***************************************************************************
 *            Utils.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include "Lince/Utils.h"

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <boost/filesystem/fstream.hpp>
#include <vector>

#include "libtorrent/http_connection.hpp"
#include "libtorrent/connection_queue.hpp"

#include <boost/version.hpp>

fs::path Lince::build_filename(const fs::path &p, const std::string &s)
{
	return fs::path(p/s);
}

std::string Lince::build_filename_as_string(const fs::path &p, const std::string &s)
{
	return fs::path(p/s).native_directory_string();
}

fs::path Lince::get_base_directory()
{
	boost::filesystem::path configDirPath;

#if !(defined(_WIN32) && defined(__MINGW32__))
	configDirPath = std::string(getenv("HOME")) +\
		std::string("/.") + std::string("lince");

#else
	configDirPath = ".";

#endif
	if (!boost::filesystem::exists(configDirPath))
	{
		boost::filesystem::create_directory(configDirPath);
	}

	return configDirPath;
}

fs::path Lince::get_torrents_base_directory()
{
	boost::filesystem::path torrentBaseDirectory;
				
#if !(defined(_WIN32) && defined(__MINGW32__))
	torrentBaseDirectory = get_base_directory() / std::string("torrents");

#else
	torrentBaseDirectory = std::string("torrents");

#endif
	//Check that exists
	if (!boost::filesystem::exists(torrentBaseDirectory))
	{
		boost::filesystem::create_directory(torrentBaseDirectory);
	}
	
	return torrentBaseDirectory;
}

fs::path Lince::get_resume_data_base_directory()
{
	boost::filesystem::path resumeBaseDirectory;
	
#if !(defined(_WIN32) && defined(__MINGW32__))
	resumeBaseDirectory = get_base_directory() / std::string("resume"); 

#else
	resumeBaseDirectory = std::string("resume");

#endif
	//Check that exists
	if (!boost::filesystem::exists(resumeBaseDirectory))
	{
		boost::filesystem::create_directory(resumeBaseDirectory);
	}
	
	return resumeBaseDirectory;
}

int Lince::to_int(const std::string& a)
{
	return static_cast<int>(atoi(a.c_str()));
}

boost::int64_t Lince::to_int64(const std::string& a)
{
	return static_cast<boost::int64_t>(atoll(a.c_str())); 
}

void Lince::save_entry (const fs::path &p, const libtorrent::entry &e)
{
	fs::ofstream out(p);	
	out.unsetf(std::ios_base::skipws);
	bencode(std::ostream_iterator<char>(out), e);
	out.close();
}

bool Lince::load_entry(const fs::path &file, libtorrent::entry &e)
{
	bool ret = true;
	fs::ifstream in(file, std::ios_base::binary);
	in.unsetf(std::ios_base::skipws);
	try
	{
		e = libtorrent::bdecode(std::istream_iterator<char>(in),
			std::istream_iterator<char>());
	}
	catch (libtorrent::invalid_encoding& ex)
	{
		ret = false;
	}
	in.close();

	return ret;
}

bool Lince::str_has_prefix(const std::string& orig, const std::string& prefix)
{
	if (prefix.length() > orig.length())
		return false;
	
	if (!orig.compare(0, prefix.length(), prefix))
		return true;
	
	return false;
}

bool Lince::str_has_suffix(const std::string& orig, const std::string& suffix)
{
	if (suffix.length() > orig.length())
		return false;
	
	if (!orig.compare(orig.length() - suffix.length(), suffix.length(), suffix))
		return true;
	
	return false;
	
}

std::string Lince::size_to_string(libtorrent::size_type value)
{
	std::ostringstream number;
		
	number.setf(std::ios::showpoint | std::ios::fixed);
		
	// Bytes
	if ( value < 1024 ){
		number.unsetf(std::ios::showpoint | std::ios::fixed);
		number << static_cast<int>(value);
		number << " B";
	}
	// KiloBytes
	else if ( value < 1048576){
		number << std::setprecision(2) << static_cast<float>(value/1024.0);
		number << " KB";
	}
	// MegaBytes
	else if ( value < 1073741824){
		number << std::setprecision(2) << static_cast<float>(value/1048576.0);
		number << " MB";
	}
	// Gigabytes
	else{
		number << std::setprecision(2) << static_cast<float>(value/1073741824.0);
		number << " GB";
	}
		
	return number.str();
}

std::string Lince::time_to_string (boost::int64_t _value)
{
	double value = static_cast<double>(_value);
	std::ostringstream number;
		
	// Seconds
	if ( value < 60 )
	{
		number << static_cast<int>(value);
		number << "s";
	}
	
	// Minutes
	else if ( value < 3600 )
	{
		number << static_cast<int>(value / 60);
		number << "m";
		value -= ((static_cast<int>(value / 60)) * 60);
		number << static_cast<int>(value);
		number << "s";
	}
	
	// Hours
	else if ( value < 86400 )
	{
		number << static_cast<int>(value / 3600);
		number << "h";
		value -= ((static_cast<int>(value / 3600)) * 3600);
		number << static_cast<int>(value / 60);
		number << "m";
	}
	
	// Days
	else
	{
		number << static_cast<int>(value / 86400);
		number << "d";
		value -= ((static_cast<int>(value / 86400)) * 86400);
		number << static_cast<int>(value / 3600);
		number << "h";
		value -= ((static_cast<int>(value / 3600)) * 3600);
		number << static_cast<int>(value / 60);
		number << "m";
	}
		
	return number.str();
}

struct Handler
{
	std::vector<char>& content;

	void operator()(const boost::system::error_code& error, 
			const libtorrent::http_parser& parser, 
			const char* data, int& size, 
			libtorrent::http_connection &connection)
	{
		for (int i = 0; i < size && data; i++)
			content.push_back(data[i]);
	}
	Handler(std::vector<char>& c) : content(c) {}
};

void Lince::get_http_torrent(const std::string &url, const std::string &name)
{
	boost::asio::io_service ios;
	std::vector<char> buff;
	boost::shared_ptr<libtorrent::http_connection> conn;
	libtorrent::connection_queue cc(ios);
	cc.limit(1);
	conn = boost::shared_ptr<libtorrent::http_connection>
		(new libtorrent::http_connection(ios, cc, Handler(buff), false));
	conn->get(url);
	ios.run();
	conn->close();

	if (!buff.empty())
	{
		boost::filesystem::ofstream out;
		out.open(name.c_str(), std::ios_base::binary);
		out.write(&buff[0], buff.size());
		out.close();
	}
}
